'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _fetch = require('./fetch');

var _fetch2 = _interopRequireDefault(_fetch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class ROXEmbedded {
  /**
   * Initialize the plugin.
   * @param {Object} options
   * @param {String} options.app Your app / environment key (see Install Instruction).
   * @param {String} options.platform Your platform of choice, if unsure, use 'Browser'.
   * @param {String} [options.outputDir] Optional. The output directory where embedded content will temporaily be written to. Defaults to your current working direcotry.
   */
  constructor(options) {
    ROXEmbedded._validate(options);
    this.options = options;
    this.output = (0, _path.resolve)(process.cwd(), this.options.outputDir || '', '~rox.embedded');
  }

  static _validate(options) {
    if (!options || options.constructor !== Object) {
      throw new Error('You must pass an options object to the constructor.');
    }
    if (!options.app || options.app.constructor !== String) {
      throw new Error("You must set the 'app' key with your setup key given by Install Instructions.");
    }
    if (!options.platform || options.platform.constructor !== String) {
      throw new Error("You must pass the 'platform' key. It can be any custom string, but we recommend using 'Browser' for Web Applications.");
    }
  }

  isWebpack5(compiler) {
    if (compiler && compiler.webpack && compiler.webpack.version && Number(compiler.webpack.version.charAt(0)) >= 5) {
      return true;
    }

    return false;
  }

  apply(compiler) {
    this.compiler = compiler;

    if (this.isWebpack5(compiler)) {
      const plugin = new _webpack2.default.WatchIgnorePlugin({ paths: [this.output] });
      plugin.apply(compiler);
      compiler.hooks.beforeRun.tapAsync('WatchIgnorePlugin', this.beforeCompilation.bind(this));
      compiler.hooks.watchRun.tapAsync('WatchIgnorePlugin', this.beforeCompilation.bind(this));
      compiler.hooks.afterCompile.tapAsync('WatchIgnorePlugin', this.afterCompilation.bind(this));
      compiler.hooks.done.tapAsync('WatchIgnorePlugin', this.onDone.bind(this));

      return;
    }

    compiler.apply(new _webpack2.default.WatchIgnorePlugin([this.output]));
    compiler.plugin('before-run', this.beforeCompilation.bind(this));
    compiler.plugin('watch-run', this.beforeCompilation.bind(this));
    compiler.plugin('after-compile', this.afterCompilation.bind(this));
    compiler.plugin('done', this.onDone.bind(this));
  }

  performStringInjection(json) {
    if (this.isWebpack5(this.compiler)) {
      const plugin = new _webpack2.default.DefinePlugin({
        $__ROX_EMBEDDED_CONTENT: json || null,
        $__ROX_EMBEDDED_OUTPUT_PATH: JSON.stringify(this.output)
      });

      plugin.apply(this.compiler);

      return;
    }

    this.compiler.apply(new _webpack2.default.DefinePlugin({
      $__ROX_EMBEDDED_CONTENT: json || null,
      $__ROX_EMBEDDED_OUTPUT_PATH: JSON.stringify(this.output)
    }));
  }

  beforeCompilation(compiler, done) {
    if (this.request) {
      return this.request.then(done);
    }
    console.log('[ROX] Compilation is starting. Fetching embedded data...');
    this.request = (0, _fetch2.default)(this.options.app, this.options.platform).then(data => {
      const json = JSON.stringify(data);
      console.log(`[ROX] Success. Writing configuration to '${this.output}'.`);
      _fs2.default.writeFileSync(this.output, json);
      return json;
    }).catch(err => {
      if (err & err.message) {
        console.error('[ROX] ' + err.message);
      } else {
        console.error('[ROX] An error occured while fetching data. Please check your internet connection and/or try again later.');
      }
      console.log('[ROX] The operation failed. Embedded configuration may not be present in this build.');
    }).then(data => this.performStringInjection(data));
    return this.request.then(done);
  }

  afterCompilation(compilation, callback) {
    if (this.request) {
      return this.request.then(callback);
    }
    callback();
  }

  onDone(stats) {
    this.request = null;
    try {
      _fs2.default.unlinkSync(this.output);
    } catch (e) {}
  }
}
exports.default = ROXEmbedded;
module.exports = exports.default;